package ui.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.QuadCurve2D;
import java.util.ArrayList;
import java.awt.Point;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import com.sun.xml.internal.bind.v2.runtime.unmarshaller.XsiNilLoader.Array;

import classes.CpsObject;
import ui.controller.Control;
import ui.model.Model;

class UnitGraph extends JPanel implements MouseListener, MouseMotionListener, ComponentListener {

	private final int NUMBER = 50;
	private static final long serialVersionUID = 1L;
	private Control controller;
	private Model model;
	private Graphics2D g2;
	private CubicCurve2D c = new CubicCurve2D.Double();
	Point[] pointList = new Point[NUMBER];

	private boolean pointDrag = false;
	private int tempP;
	private int x1, x2, y1, y2, ctrlx1, ctrly1, ctrlx2, ctrly2;
	private double ratio = this.getHeight();

	public UnitGraph(final Model model, Control control) {

		this.controller = control;
		this.model = model;
		for (int i = 0; i < pointList.length; i++) {
			pointList[i] = new Point(0, 0);
		}
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addComponentListener(this);
	}

	/**
	 * Paints all Components on the Canvas
	 * 
	 * @param Graphics
	 * 
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		g2 = (Graphics2D) g;
		RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHints(rh);
		g2.setStroke(new BasicStroke(1));

		g2.setColor(Color.LIGHT_GRAY);
		for (int i = 0; i < pointList.length; i++) {
			g2.drawLine((int) pointList[i].getX(), 0, (int) pointList[i].getX(), this.getHeight());
		}

		g2.setColor(Color.BLACK);
		g2.setStroke(new BasicStroke(2));
		for (int i = 0; i < pointList.length - 1; i++) {
			x1 = (int) pointList[i].getX();
			y1 = (int) pointList[i].getY();
			x2 = (int) pointList[i + 1].getX();
			y2 = (int) pointList[i + 1].getY();
			ctrlx1 = (int) pointList[i].getX()+((int) pointList[i+1].getX()-(int) pointList[i].getX())/2;
			ctrlx2 = (int) pointList[i+1].getX()-((int) pointList[i+1].getX()-(int) pointList[i].getX())/2;
			if(y1<y2){
				ctrly1 = (int) pointList[i].getY()+((int) pointList[i+1].getY()-(int) pointList[i].getY())/10;
				ctrly2 = (int) pointList[i+1].getY()-((int) pointList[i+1].getY()-(int) pointList[i].getY())/10;
			} else{
				ctrly1 = (int) pointList[i].getY()-((int) pointList[i].getY()-(int) pointList[i+1].getY())/10;
				ctrly2 = (int) pointList[i+1].getY()+((int) pointList[i].getY()-(int) pointList[i+1].getY())/10;
			}
			
			c.setCurve(x1, y1, ctrlx1, ctrly1, ctrlx2, ctrly2, x2, y2);
			
			
			//draw the curve
			g2.draw(c);
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		if (pointDrag && e.getY() >= this.getHeight() / 3 && tempP != -1) {
			pointList[tempP].setLocation(pointList[tempP].getX(), e.getY());
			if(tempP!=0){
				pointList[tempP-1].setLocation(pointList[tempP-1].getX(), pointList[tempP-1].getY()+(pointList[tempP].getY()-e.getY())/2);
			}
			if(tempP!=NUMBER-1){
				pointList[tempP+1].setLocation(pointList[tempP+1].getX(), pointList[tempP+1].getY()+(pointList[tempP].getY()-e.getY())/2);
			}
		} else if(tempP != -1){
			pointList[tempP].setLocation(pointList[tempP].getX(), this.getHeight()/3);
		}
		repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
	}

	@Override
	public void mousePressed(MouseEvent e) {
		for (int i = 0; i < pointList.length; i++) {
			if (e.getX() - this.getWidth() / NUMBER / 2 <= pointList[i].getX()
					&& e.getX() + this.getWidth() / NUMBER / 2 >= pointList[i].getX()
					&& e.getY() - 10 < pointList[i].getY() && e.getY() + 10 > pointList[i].getY()) {
				tempP = i;
				pointDrag = true;
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		if (pointDrag) {
			pointDrag = false;
			tempP = -1;
		}
	}

	// resize listener
	public void componentResized(ComponentEvent e) {
		if(pointList[0].getY() != 0)
		for (int i = 0; i < pointList.length; i++) {
			pointList[i] = new Point((i) * this.getWidth() / (NUMBER - 1), (int) pointList[i].getY());
		} else {
			for (int i = 0; i < pointList.length; i++) {
				pointList[i] = new Point((i) * this.getWidth() / (NUMBER - 1), this.getHeight() / 3);
			}
		}
		repaint();
	}

	@Override
	public void componentHidden(ComponentEvent e) {
	}

	@Override
	public void componentMoved(ComponentEvent e) {

	}

	@Override
	public void componentShown(ComponentEvent e) {
	}

}